/**
 * Copyright (C) 2013  Dionis Argiri
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package hqlcriteria.expression;

import hqlcriteria.HQLCriteriaBuildContext;

/**
 * @author dionis
 */
public class Operators {
	private static class And implements Operator {
		private final Operator left;
		private final Operator right;

		public And(Operator left, Operator right) {
			this.left = left;
			this.right = right;
		}

		public String apply(HQLCriteriaBuildContext hqlCriteriaBuildContext) {
			return "(" + left.apply(hqlCriteriaBuildContext) + " and " + right.apply(hqlCriteriaBuildContext) + ")";
		}
	}
	
	private static class Or implements Operator {
		private final Operator left;
		private final Operator right;

		public Or(Operator left, Operator right) {
			this.left = left;
			this.right = right;
		}

		public String apply(HQLCriteriaBuildContext hqlCriteriaBuildContext) {
			return "(" + left.apply(hqlCriteriaBuildContext) + " or " + right.apply(hqlCriteriaBuildContext) + ")";
		}
	}
	
	public static Operator and(Operator left, Operator right){
		return new And(left, right);
	}
	
	public static Operator or(Operator left, Operator right){
		return new Or(left, right);
	}

    private static class Not implements Operator {
        private final Expression target;

        public Not(Expression target) {
            this.target = target;
        }

        public String apply(HQLCriteriaBuildContext hqlCriteriaBuildContext) {
            return " not (" + target.apply(hqlCriteriaBuildContext) + ")";
        }
    }

    public static Operator not(String target) {
        return new Not(Value.of(target));
    }

    public static Operator not(Expression target) {
        return new Not(target);
    }
}
